@@ -21,6 +21,7 @@
 #include "math-buttons.h"
 #include "financial.h"
 #include "currency.h"
+#include "mp-serializer.h"
 
 enum {
     PROP_0,
@@ -335,7 +336,7 @@ set_tint (GtkWidget *widget, GdkColor *tint, gint alpha)
         color.red = (style->bg[j].red * (10 - alpha) + tint->red * alpha) / 10;
         color.green = (style->bg[j].green * (10 - alpha) + tint->green * alpha) / 10;
         color.blue = (style->bg[j].blue * (10 - alpha) + tint->blue * alpha) / 10;
-        gdk_colormap_alloc_color(gdk_colormap_get_system(), &color, FALSE, TRUE);
+
         gtk_widget_modify_bg(widget, j, &color);
     }
 }
@@ -422,10 +423,10 @@ update_angle_label (MathButtons *buttons)
             mp_set_from_mp(&x, &input);
             mp_set_from_integer(mp_is_negative(&input) ? -1 : 1, &fraction);
         }
-        mp_cast_to_string(&input, 10, 10, 2, false, '.', input_text, 1024);
+        mp_serializer_to_specific_string(&input, 10, 2, false, true, input_text, 1024);
 
         mp_multiply_integer(&fraction, 360, &output);
-        mp_cast_to_string(&output, 10, 10, 2, false, '.', output_text, 1024);
+        mp_serializer_to_specific_string(&output, 10, 2, false, true, output_text, 1024);
         label = g_strdup_printf(_("%s radians = %s degrees"), input_text, output_text);
         break;
     case MP_GRADIANS:
@@ -442,10 +443,10 @@ update_angle_label (MathButtons *buttons)
             mp_set_from_integer(mp_is_negative(&input) ? -1 : 1, &fraction);
         }
 
-        mp_cast_to_string(&input, 10, 10, 2, false, '.', input_text, 1024);
+        mp_serializer_to_specific_string(&input, 10, 2, false, true, input_text, 1024);
 
         mp_multiply_integer(&fraction, 360, &output);
-        mp_cast_to_string(&output, 10, 10, 2, false, '.', output_text, 1024);
+        mp_serializer_to_specific_string(&output, 10, 2, false, true, output_text, 1024);
         label = g_strdup_printf(_("%s gradians = %s degrees"), input_text, output_text);
         break;
     }
@@ -543,8 +544,8 @@ update_currency_label(MathButtons *buttons)
         const char *source_symbol, *target_symbol;
         int i;
 
-        mp_cast_to_string(&x, 10, 10, 2, false, '.', input_text, 1024);
-        mp_cast_to_string(&value, 10, 10, 2, false, '.', output_text, 1024);
+        mp_serializer_to_specific_string(&x, 10, 2, false, true, input_text, 1024);
+        mp_serializer_to_specific_string(&value, 10, 2, false, true, output_text, 1024);
 
         for (i = 0; strcmp(math_equation_get_source_currency(buttons->priv->equation), currency_names[i].short_name) != 0; i++);
         source_symbol = currency_names[i].symbol;
@@ -800,7 +801,7 @@ load_mode(MathButtons *buttons, ButtonMode mode)
         g_clear_error(&error);
     }
     *panel = GET_WIDGET(builder, "button_panel");
-    gtk_box_pack_end(GTK_BOX(buttons), *panel, FALSE, TRUE, 0);
+    gtk_box_pack_end(GTK_BOX(buttons), *panel, TRUE, TRUE, 0);
 
     /* Configure buttons */
     for (i = 0; button_data[i].widget_name != NULL; i++) {
@@ -854,26 +855,18 @@ load_mode(MathButtons *buttons, ButtonMode mode)
         name = g_strdup_printf("calc_%d_button", i);
         button = GET_WIDGET(builder, name);
         if (button) {
-            gchar buffer[7];
-            gint len;
-
             g_object_set_data(G_OBJECT(button), "calc_digit", GINT_TO_POINTER(i));
             set_tint(button, &buttons->priv->color_numbers, 1);
-            len = g_unichar_to_utf8(math_equation_get_digit_text(buttons->priv->equation, i), buffer);
-            buffer[len] = '\0';
-            gtk_button_set_label(GTK_BUTTON(button), buffer);
+            gtk_button_set_label(GTK_BUTTON(button), math_equation_get_digit_text(buttons->priv->equation, i));
         }
         g_free(name);
     }
     widget = GET_WIDGET(builder, "calc_numeric_point_button");
     if (widget) {
-        gchar buffer[7];
-        gint len;
-        len = g_unichar_to_utf8(math_equation_get_numeric_point_text(buttons->priv->equation), buffer);
-        buffer[len] = '\0';
-        gtk_button_set_label(GTK_BUTTON(widget), buffer);
+        MpSerializer *serializer = math_equation_get_serializer(buttons->priv->equation);
+        gtk_button_set_label(GTK_BUTTON(widget), mp_serializer_get_numeric_point_text(serializer));
     }
-
+  
     widget = GET_WIDGET(builder, "calc_superscript_button");
     if (widget) {
         buttons->priv->superscript_toggles = g_list_append(buttons->priv->superscript_toggles, widget);
@@ -1204,7 +1197,8 @@ make_register_menu_item(MathButtons *buttons, const gchar *name, const MPNumber
     GtkWidget *item, *label;
 
     if (value) {
-        display_make_number(buttons->priv->equation, text, 1024, value);
+        MpSerializer *serializer = math_equation_get_serializer(buttons->priv->equation);
+        mp_serializer_to_standard_string(serializer, value, text, 1024);
         mstr = g_strdup_printf("<span weight=\"bold\">%s</span> = %s", name, text);
     }
     else
