@@ -24,14 +24,14 @@
 #include <math.h>
 #include <errno.h>
 #include <glib.h>
-#include <langinfo.h>
-#include <locale.h>
 
 #include "math-equation.h"
 
 #include "mp.h"
 #include "mp-equation.h"
+#include "mp-serializer.h"
 #include "currency.h"
+#include "math-enums.h"
 
 
 enum {
@@ -46,7 +46,10 @@ enum {
     PROP_NUMBER_FORMAT,
     PROP_BASE,
     PROP_WORD_SIZE,
-    PROP_ANGLE_UNITS
+    PROP_ANGLE_UNITS,
+    PROP_SOURCE_CURRENCY,
+    PROP_TARGET_CURRENCY,
+    PROP_SERIALIZER
 };
 
 static GType number_mode_type, number_format_type, angle_unit_type;
@@ -69,20 +72,14 @@ struct MathEquationPrivate
 {
     GtkTextTag *ans_tag;
 
-    gint show_tsep;           /* Set if the thousands separator should be shown. */
-    gint show_zeroes;         /* Set if trailing zeroes should be shown. */
-    DisplayFormat format;     /* Number display mode. */
-    gint accuracy;            /* Number of digits to show */
     gint word_size;           /* Word size in bits */
     MPAngleUnit angle_units;  /* Units for trigonometric functions */
-    gint base;                /* Numeric base */
+    char *source_currency;
+    char *target_currency;
     NumberMode number_mode;   /* ??? */
     gboolean can_super_minus; /* TRUE if entering minus can generate a superscript minus */
 
     const char *digits[16];   /* Localized digit values */
-    const char *radix;        /* Locale specific radix string. */
-    const char *tsep;         /* Locale specific thousands separator. */
-    gint tsep_count;          /* Number of digits between separator. */
 
     GtkTextMark *ans_start, *ans_end;
 
@@ -96,11 +93,7 @@ struct MathEquationPrivate
     gboolean in_delete;
 
     MathVariables *variables;
-
-    // FIXME: Replace with GtkClipboard
-    GdkAtom clipboard_atom;   /* ??? */ // 
-    GdkAtom primary_atom;     /* ??? */ // FIXME: Is this middle click?
-    char *shelf;              /* PUT selection shelf contents. */
+    MpSerializer *serializer;
 };
 
 G_DEFINE_TYPE (MathEquation, math_equation, GTK_TYPE_TEXT_BUFFER);
@@ -150,7 +143,7 @@ reformat_ans(MathEquation *equation)
     gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(equation), &ans_start, equation->priv->ans_start);
     gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(equation), &ans_end, equation->priv->ans_end);
     orig_ans_text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(equation), &ans_start, &ans_end, FALSE);
-    display_make_number(equation, ans_text, MAX_DIGITS, &equation->priv->state.ans);
+    mp_serializer_to_standard_string(equation->priv->serializer, &equation->priv->state.ans, ans_text, MAX_DIGITS);
     if (strcmp(orig_ans_text, ans_text) != 0) {
         gint start;
 
@@ -174,7 +167,7 @@ reformat_ans(MathEquation *equation)
     g_free(orig_ans_text);
 }
 
-
+#if 0
 /* NOTE: Not efficent but easy to write */
 // FIXME: This is just a lexer - use the same lexer as the solver
 static void
@@ -321,7 +314,7 @@ reformat_separators(MathEquation *equation)
     g_free(text);
 #endif
 }
-
+#endif
 
 static void
 reformat_display(MathEquation *equation, gint old_base)
@@ -330,10 +323,10 @@ reformat_display(MathEquation *equation, gint old_base)
     reformat_ans(equation);
 
     /* Add/remove base suffixes if have changed base */
-    reformat_base(equation, old_base);
+    //reformat_base(equation, old_base);
 
     /* Add/remove thousands separators */
-    reformat_separators(equation);
+    //reformat_separators(equation);
 }
 
 
@@ -455,6 +448,37 @@ apply_state(MathEquation *equation, MathEquationState *state)
 }
 
 
+void
+math_equation_copy(MathEquation *equation)
+{
+    GtkTextIter start, end;
+    gchar *text;
+
+    if (!gtk_text_buffer_get_selection_bounds(GTK_TEXT_BUFFER(equation), &start, &end))
+        gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(equation), &start, &end);
+
+    text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(equation), &start, &end, FALSE);
+    gtk_clipboard_set_text(gtk_clipboard_get(GDK_NONE), text, -1);
+    g_free (text);
+}
+
+
+static void
+on_paste(GtkClipboard *clipboard, const gchar *text, gpointer data)
+{
+    MathEquation *equation = data;
+    if (text != NULL)
+        math_equation_insert (equation, text);
+}
+
+
+void
+math_equation_paste(MathEquation *equation)
+{
+    gtk_clipboard_request_text(gtk_clipboard_get(GDK_NONE), on_paste, equation);
+}
+
+
 void
 math_equation_undo(MathEquation *equation)
 {
@@ -512,20 +536,14 @@ math_equation_get_digit_text(MathEquation *equation, guint digit)
 }
 
 
-const gchar *
-math_equation_get_numeric_point_text(MathEquation *equation)
-{
-    return equation->priv->radix;
-}
-
-
 void
 math_equation_set_accuracy(MathEquation *equation, gint accuracy)
 {
-    if (equation->priv->accuracy == accuracy)
+    gint old_accuracy = mp_serializer_get_accuracy(equation->priv->serializer);
+    if (old_accuracy == accuracy)
         return;
-    equation->priv->accuracy = accuracy;
-    reformat_display(equation, equation->priv->base);
+    mp_serializer_set_accuracy(equation->priv->serializer, accuracy);
+    reformat_display(equation, mp_serializer_get_base(equation->priv->serializer));
     g_object_notify(G_OBJECT(equation), "accuracy");
 }
 
@@ -533,17 +551,18 @@ math_equation_set_accuracy(MathEquation *equation, gint accuracy)
 gint
 math_equation_get_accuracy(MathEquation *equation)
 {
-    return equation->priv->accuracy;
+    return mp_serializer_get_accuracy(equation->priv->serializer);
 }
 
 
 void
 math_equation_set_show_thousands_separators(MathEquation *equation, gboolean visible)
 {
-    if ((equation->priv->show_tsep && visible) || (!equation->priv->show_tsep && !visible))
+    gboolean old_visible = mp_serializer_get_show_thousands_separators(equation->priv->serializer);
+    if (old_visible == visible)
         return;
-    equation->priv->show_tsep = visible;
-    reformat_display(equation, equation->priv->base);
+    mp_serializer_set_show_thousands_separators(equation->priv->serializer, visible);
+    reformat_display(equation, mp_serializer_get_base(equation->priv->serializer));
     g_object_notify(G_OBJECT(equation), "show-thousands-separators");
 }
 
@@ -551,17 +570,18 @@ math_equation_set_show_thousands_separators(MathEquation *equation, gboolean vis
 gboolean
 math_equation_get_show_thousands_separators(MathEquation *equation)
 {
-    return equation->priv->show_tsep;
+    return mp_serializer_get_show_thousands_separators(equation->priv->serializer);
 }
 
 
 void
 math_equation_set_show_trailing_zeroes(MathEquation *equation, gboolean visible)
 {
-    if ((equation->priv->show_zeroes && visible) || (!equation->priv->show_zeroes && !visible))
+    gboolean old_visible = mp_serializer_get_show_trailing_zeroes(equation->priv->serializer);
+    if (old_visible == visible)
         return;
-    equation->priv->show_zeroes = visible;
-    reformat_display(equation, equation->priv->base);
+    mp_serializer_set_show_trailing_zeroes(equation->priv->serializer, visible);
+    reformat_display(equation, mp_serializer_get_base(equation->priv->serializer));
     g_object_notify(G_OBJECT(equation), "show-trailing-zeroes");
 }
 
@@ -569,18 +589,19 @@ math_equation_set_show_trailing_zeroes(MathEquation *equation, gboolean visible)
 gboolean
 math_equation_get_show_trailing_zeroes(MathEquation *equation)
 {
-    return equation->priv->show_zeroes;
+    return mp_serializer_get_show_trailing_zeroes(equation->priv->serializer);
 }
 
 
 void
 math_equation_set_number_format(MathEquation *equation, DisplayFormat format)
 {
-    if (equation->priv->format == format)
+    DisplayFormat old_format = mp_serializer_get_number_format(equation->priv->serializer);
+    if (old_format == format)
         return;
 
-    equation->priv->format = format;
-    reformat_display(equation, equation->priv->base);
+    mp_serializer_set_number_format(equation->priv->serializer, format);
+    reformat_display(equation, mp_serializer_get_base(equation->priv->serializer));
     g_object_notify(G_OBJECT(equation), "number-format");
 }
 
@@ -588,20 +609,19 @@ math_equation_set_number_format(MathEquation *equation, DisplayFormat format)
 DisplayFormat
 math_equation_get_number_format(MathEquation *equation)
 {
-    return equation->priv->format;
+    return mp_serializer_get_number_format(equation->priv->serializer);
 }
 
 
 void
 math_equation_set_base(MathEquation *equation, gint base)
 {
-    gint old_base;
+    gint old_base = mp_serializer_get_base(equation->priv->serializer);
 
-    if (equation->priv->base == base)
+    if (old_base == base)
         return;
 
-    old_base = equation->priv->base;
-    equation->priv->base = base;
+    mp_serializer_set_base(equation->priv->serializer, base);
     reformat_display(equation, old_base);
     g_object_notify(G_OBJECT(equation), "base");
 }
@@ -610,7 +630,7 @@ math_equation_set_base(MathEquation *equation, gint base)
 gint
 math_equation_get_base(MathEquation *equation)
 {
-    return equation->priv->base;
+    return mp_serializer_get_base(equation->priv->serializer);
 }
 
 
@@ -648,6 +668,49 @@ math_equation_get_angle_units(MathEquation *equation)
 }
 
 
+void
+math_equation_set_source_currency(MathEquation *equation, const gchar *currency)
+{
+    // FIXME: Pick based on locale  
+    if (!currency || currency[0] == '\0')
+        currency = currency_names[0].short_name;
+
+    if (strcmp(equation->priv->source_currency, currency) == 0)
+        return;
+    g_free(equation->priv->source_currency);
+    equation->priv->source_currency = g_strdup(currency);
+    g_object_notify(G_OBJECT(equation), "source-currency");
+}
+
+const gchar *
+math_equation_get_source_currency(MathEquation *equation)
+{
+    return equation->priv->source_currency;
+}
+
+
+void
+math_equation_set_target_currency(MathEquation *equation, const gchar *currency)
+{
+    // FIXME: Pick based on locale  
+    if (!currency || currency[0] == '\0')
+        currency = currency_names[0].short_name;
+
+    if (strcmp(equation->priv->target_currency, currency) == 0)
+        return;
+    g_free(equation->priv->target_currency);
+    equation->priv->target_currency = g_strdup(currency);
+    g_object_notify(G_OBJECT(equation), "target-currency");
+}
+
+
+const gchar *
+math_equation_get_target_currency(MathEquation *equation)
+{
+    return equation->priv->target_currency;
+}
+
+
 void
 math_equation_set_status(MathEquation *equation, const gchar *status)
 {
@@ -724,12 +787,17 @@ math_equation_get_number(MathEquation *equation, MPNumber *z)
     gboolean result;
 
     text = math_equation_get_display(equation);
-    result = !mp_set_from_string(text, equation->priv->base, z);
+    result = !mp_serializer_from_string(equation->priv->serializer, text, z);
     g_free (text);
 
     return result;
 }
 
+MpSerializer*
+math_equation_get_serializer(MathEquation *equation)
+{
+    return equation->priv->serializer;
+}
 
 void
 math_equation_set_number_mode(MathEquation *equation, NumberMode mode)
@@ -758,99 +826,6 @@ math_equation_get_answer(MathEquation *equation)
 }
 
 
-void
-math_equation_copy(MathEquation *equation)
-{
-    gchar *string = NULL;
-    GtkTextIter start, end;
-
-    if (!gtk_text_buffer_get_selection_bounds(GTK_TEXT_BUFFER(equation), &start, &end))
-        gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(equation), &start, &end);
-
-    string = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(equation), &start, &end, FALSE);
-
-    if (equation->priv->shelf != NULL)
-        g_free(equation->priv->shelf);
-    equation->priv->shelf = g_locale_from_utf8(string, strlen(string), NULL, NULL, NULL);
-
-    gtk_clipboard_set_text(gtk_clipboard_get(equation->priv->clipboard_atom), equation->priv->shelf, -1);
-
-    g_free(string);
-}
-
-
-static void
-on_paste(GtkClipboard *clipboard, const gchar *text, MathEquation *equation)
-{
-    const char *input;
-    char c, *output, *clean_text;
-  
-    /* Copy input to modify, no operation can make the clean string longer than
-     * the original string */
-    clean_text = strdup(text);
-
-    output = clean_text;
-    for (input = text; *input; input++) {
-        /* If the clipboard buffer contains any occurances of the "thousands
-         * separator", remove them.
-         */
-        if (equation->priv->tsep[0] != '\0' && strncmp(input, equation->priv->tsep, strlen(equation->priv->tsep)) == 0) {
-            input += strlen(equation->priv->tsep) - 1;
-            continue;
-        }
-
-        /* Replace radix with "." */
-        else if (strncmp(input, equation->priv->radix, strlen(equation->priv->radix)) == 0) {
-            input += strlen(equation->priv->radix) - 1;
-            c = '.';
-        }
-
-        /* Replace tabs with spaces */
-        else if (*input == '\t') {
-            c = ' ';
-        }
-
-        /* Terminate on newlines */
-        else if (*input == '\r' || *input == '\n') {
-            c = '\0';
-        }
-
-        /* If an "A", "B", "C", "D" or "F" character is encountered, it
-         * will be converted to its lowercase equivalent. If an "E" is
-         * found,  and the next character is a "-" or a "+", then it
-         * remains as an upper case "E" (it's assumed to be a possible
-         * exponential number), otherwise its converted to a lower case
-         * "e". See bugs #455889 and #469245 for more details.
-         */
-        else if (*input >= 'A' && *input <= 'F') {
-            c = *input;
-            if (*input == 'E') {
-                if (*(input+1) != '-' && *(input+1) != '+')
-                    c = tolower(*input);
-            }
-            else
-                c = tolower(*input);
-        }
-
-        else
-            c = *input;
-
-        *output++ = c;
-    }
-    *output++ = '\0';
-
-    math_equation_insert(equation, clean_text);
-}
-
-
-void
-math_equation_paste(MathEquation *equation)
-{
-    gtk_clipboard_request_text(gtk_clipboard_get(equation->priv->clipboard_atom),
-                               (GtkClipboardTextReceivedFunc)on_paste, equation);
-}
-
-
 void
 math_equation_store(MathEquation *equation, const gchar *name)
 {
@@ -886,7 +861,7 @@ math_equation_set_number(MathEquation *equation, const MPNumber *x)
     GtkTextIter start, end;
 
     /* Show the number in the user chosen format */
-    display_make_number(equation, text, MAX_DIGITS, x);
+    mp_serializer_to_standard_string(equation->priv->serializer, x, text, MAX_DIGITS);
     gtk_text_buffer_set_text(GTK_TEXT_BUFFER(equation), text, -1);
     mp_set_from_mp(x, &equation->priv->state.ans);
 
@@ -950,7 +925,7 @@ math_equation_insert_digit(MathEquation *equation, guint digit)
 void
 math_equation_insert_numeric_point(MathEquation *equation)
 {
-    math_equation_insert(equation, math_equation_get_numeric_point_text(equation));
+    math_equation_insert(equation, mp_serializer_get_numeric_point_text(equation->priv->serializer));
 }
 
 
@@ -958,7 +933,7 @@ void
 math_equation_insert_number(MathEquation *equation, const MPNumber *x)
 {
     char text[MAX_DIGITS];
-    display_make_number(equation, text, MAX_DIGITS, x);
+    mp_serializer_to_standard_string(equation->priv->serializer, x, text, MAX_DIGITS);
     math_equation_insert(equation, text);
 }
 
@@ -1048,17 +1023,7 @@ set_variable(const char *name, const MPNumber *x, void *data)
 static int
 convert(const MPNumber *x, const char *x_units, const char *z_units, MPNumber *z, void *data)
 {   
-    /* Update currency if necessary */
-    if (currency_rates_needs_update())
-        currency_download_rates();
-    currency_load_rates();
-    if (currency_get_index(x_units) >= 0 && currency_get_index(z_units) >= 0)
-    {
-        currency_convert(x, currency_get_index(x_units), currency_get_index(z_units), z);
-        return 1;
-    }
-
-    return 0;
+    return currency_convert(x, x_units, z_units, z);
 }
 
 
@@ -1068,7 +1033,7 @@ parse(MathEquation *equation, const char *text, MPNumber *z, char **error_token)
     MPEquationOptions options;
 
     memset(&options, 0, sizeof(options));
-    options.base = equation->priv->base;
+    options.base = mp_serializer_get_base(equation->priv->serializer);
     options.wordlen = equation->priv->word_size;
     options.angle_units = equation->priv->angle_units;
     options.variable_is_defined = variable_is_defined;
@@ -1187,7 +1152,7 @@ math_equation_factorize(MathEquation *equation)
         MPNumber *n;
 
         n = factor->data;
-        display_make_number(equation, temp, MAX_DIGITS, n);
+        mp_serializer_to_standard_string(equation->priv->serializer, n, temp, MAX_DIGITS);
         g_string_append(text, temp);
         if (factor->next)
             g_string_append(text, "×");
@@ -1293,25 +1258,6 @@ math_equation_toggle_bit(MathEquation *equation, guint bit)
 }
 
 
-/* Convert MP number to character string. */
-//FIXME: What to do with this?
-void
-display_make_number(MathEquation *equation, char *target, int target_len, const MPNumber *x)
-{
-    switch(equation->priv->format) {
-    case FIX:
-        mp_cast_to_string(x, equation->priv->base, equation->priv->base, equation->priv->accuracy, !equation->priv->show_zeroes, target, target_len);
-        break;
-    case SCI:
-        mp_cast_to_exponential_string(x, equation->priv->base, equation->priv->base, equation->priv->accuracy, !equation->priv->show_zeroes, false, target, target_len);
-        break;
-    case ENG:
-        mp_cast_to_exponential_string(x, equation->priv->base, equation->priv->base, equation->priv->accuracy, !equation->priv->show_zeroes, true, target, target_len);
-        break;
-    }
-}
-
-
 static void
 math_equation_set_property(GObject      *object,
                            guint         prop_id,
@@ -1324,35 +1270,41 @@ math_equation_set_property(GObject      *object,
 
     switch (prop_id) {
     case PROP_STATUS:
-      math_equation_set_status(self, g_value_get_string(value));
-      break;
+        math_equation_set_status(self, g_value_get_string(value));
+        break;
     case PROP_DISPLAY:
-      math_equation_set(self, g_value_get_string(value));
-      break;
+        math_equation_set(self, g_value_get_string(value));
+        break;
     case PROP_NUMBER_MODE:
-      math_equation_set_number_mode(self, g_value_get_int(value));
-      break;
+        math_equation_set_number_mode(self, g_value_get_int(value));
+        break;
     case PROP_ACCURACY:
-      math_equation_set_accuracy(self, g_value_get_int(value));
-      break;
+        math_equation_set_accuracy(self, g_value_get_int(value));
+        break;
     case PROP_SHOW_THOUSANDS_SEPARATORS:
-      math_equation_set_show_thousands_separators(self, g_value_get_boolean(value));
-      break;
+        math_equation_set_show_thousands_separators(self, g_value_get_boolean(value));
+        break;
     case PROP_SHOW_TRAILING_ZEROES:
-      math_equation_set_show_trailing_zeroes(self, g_value_get_boolean(value));
-      break;
+        math_equation_set_show_trailing_zeroes(self, g_value_get_boolean(value));
+        break;
     case PROP_NUMBER_FORMAT:
-      math_equation_set_number_format(self, g_value_get_int(value));
-      break;
+        math_equation_set_number_format(self, g_value_get_int(value));
+        break;
     case PROP_BASE:
-      math_equation_set_base(self, g_value_get_int(value));
-      break;
+        math_equation_set_base(self, g_value_get_int(value));
+        break;
     case PROP_WORD_SIZE:
-      math_equation_set_word_size(self, g_value_get_int(value));
-      break;
+        math_equation_set_word_size(self, g_value_get_int(value));
+        break;
     case PROP_ANGLE_UNITS:
-      math_equation_set_angle_units(self, g_value_get_int(value));
-      break;
+        math_equation_set_angle_units(self, g_value_get_int(value));
+        break;
+    case PROP_SOURCE_CURRENCY:
+        math_equation_set_source_currency(self, g_value_get_string(value));
+        break;
+    case PROP_TARGET_CURRENCY:
+        math_equation_set_target_currency(self, g_value_get_string(value));
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -1373,42 +1325,51 @@ math_equation_get_property(GObject    *object,
 
     switch (prop_id) {
     case PROP_STATUS:
-      g_value_set_string(value, self->priv->state.status);
-      break;
+        g_value_set_string(value, self->priv->state.status);
+        break;
     case PROP_DISPLAY:
-      text = math_equation_get_display(self);      
-      g_value_set_string(value, text);
-      g_free(text);
-      break;
+        text = math_equation_get_display(self);      
+        g_value_set_string(value, text);
+        g_free(text);
+        break;
     case PROP_EQUATION:
-      text = math_equation_get_equation(self);
-      g_value_set_string(value, text);
-      g_free(text);
-      break;
+        text = math_equation_get_equation(self);
+        g_value_set_string(value, text);
+        g_free(text);
+        break;
     case PROP_NUMBER_MODE:
-      g_value_set_enum(value, self->priv->number_mode);
-      break;
+        g_value_set_enum(value, self->priv->number_mode);
+        break;
     case PROP_ACCURACY:
-      g_value_set_int(value, self->priv->accuracy);
-      break;
+        g_value_set_int(value, mp_serializer_get_accuracy(self->priv->serializer));
+        break;
     case PROP_SHOW_THOUSANDS_SEPARATORS:
-      g_value_set_boolean(value, self->priv->show_tsep);
-      break;
+        g_value_set_boolean(value, mp_serializer_get_show_thousands_separators(self->priv->serializer));
+        break;
     case PROP_SHOW_TRAILING_ZEROES:
-      g_value_set_boolean(value, self->priv->show_zeroes);
-      break;
+        g_value_set_boolean(value, mp_serializer_get_show_trailing_zeroes(self->priv->serializer));
+        break;
     case PROP_NUMBER_FORMAT:
-      g_value_set_enum(value, self->priv->format);
-      break;
+        g_value_set_enum(value, mp_serializer_get_number_format(self->priv->serializer));
+        break;
     case PROP_BASE:
-      g_value_set_int(value, math_equation_get_base(self));
-      break;
+        g_value_set_int(value, math_equation_get_base(self));
+        break;
     case PROP_WORD_SIZE:
-      g_value_set_int(value, self->priv->word_size);
-      break;
+        g_value_set_int(value, self->priv->word_size);
+        break;
     case PROP_ANGLE_UNITS:
-      g_value_set_enum(value, self->priv->angle_units);
-      break;
+        g_value_set_enum(value, self->priv->angle_units);
+        break;
+    case PROP_SOURCE_CURRENCY:
+        g_value_set_string(value, self->priv->source_currency);
+        break;
+    case PROP_TARGET_CURRENCY:
+        g_value_set_string(value, self->priv->target_currency);
+        break;
+    case PROP_SERIALIZER:
+        g_value_set_object(value, self->priv->serializer);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -1426,13 +1387,6 @@ math_equation_class_init (MathEquationClass *klass)
       {SUBSCRIPT,   "subscript",   "subscript"},
       {0, NULL, NULL}
     };
-    static GEnumValue number_format_values[] =
-    {
-      {FIX, "fixed-point", "fixed-point"},
-      {SCI, "scientific",  "scientific"},
-      {ENG, "engineering", "engineering"},
-      {0, NULL, NULL}
-    };
     static GEnumValue angle_unit_values[] =
     {
       {MP_RADIANS,  "radians",  "radians"},
@@ -1448,7 +1402,7 @@ math_equation_class_init (MathEquationClass *klass)
     g_type_class_add_private (klass, sizeof (MathEquationPrivate));
   
     number_mode_type = g_enum_register_static("NumberMode", number_mode_values);
-    number_format_type = g_enum_register_static("DisplayFormat", number_format_values);
+    number_format_type = math_display_format_get_type();
     angle_unit_type = g_enum_register_static("AngleUnit", angle_unit_values);
 
     g_object_class_install_property(object_class,
@@ -1531,6 +1485,27 @@ math_equation_class_init (MathEquationClass *klass)
                                                       angle_unit_type,
                                                       MP_DEGREES,
                                                       G_PARAM_READWRITE));
+    g_object_class_install_property(object_class,
+                                    PROP_SOURCE_CURRENCY,
+                                    g_param_spec_string("source-currency",
+                                                        "source-currency",
+                                                        "Source Currency",
+                                                        "",
+                                                        G_PARAM_READWRITE));
+    g_object_class_install_property(object_class,
+                                    PROP_TARGET_CURRENCY,
+                                    g_param_spec_string("target-currency",
+                                                        "target-currency",
+                                                        "target Currency",
+                                                        "",
+                                                        G_PARAM_READWRITE));
+    g_object_class_install_property(object_class,
+                                    PROP_SERIALIZER,
+                                    g_param_spec_object("serializer",
+                                                        "serializer",
+                                                        "Serializer",
+                                                        MP_TYPE_SERIALIZER,
+                                                        G_PARAM_READABLE));
 }
 
 
@@ -1646,7 +1621,6 @@ math_equation_init(MathEquation *equation)
     /* Digits localized for the given language */
     const char *digit_values = _("0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F");
     const char *default_digits[] = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F"};
-    gchar *radix, *tsep;
     gchar **digits;
     gboolean use_default_digits = FALSE;
     int i;
@@ -1673,29 +1647,15 @@ math_equation_init(MathEquation *equation)
     }
     g_strfreev(digits);
 
-    setlocale(LC_NUMERIC, "");
-
-    radix = nl_langinfo(RADIXCHAR);
-    equation->priv->radix = radix ? g_locale_to_utf8(radix, -1, NULL, NULL, NULL) : g_strdup(".");
-    tsep = nl_langinfo(THOUSEP);
-    equation->priv->tsep = tsep ? g_locale_to_utf8(tsep, -1, NULL, NULL, NULL) : g_strdup(",");
-
-    equation->priv->tsep_count = 3;
-  
     equation->priv->variables = math_variables_new();
 
-    // Use GtkClipboad instead
-    equation->priv->primary_atom = gdk_atom_intern("PRIMARY", FALSE);
-    equation->priv->clipboard_atom = gdk_atom_intern("CLIPBOARD", FALSE);
-
     equation->priv->state.status = g_strdup("");
-    equation->priv->show_zeroes = FALSE;
-    equation->priv->show_tsep = FALSE;
-    equation->priv->format = FIX;
-    equation->priv->accuracy = 9;
     equation->priv->word_size = 32;
     equation->priv->angle_units = MP_DEGREES;
-    equation->priv->base = 10;
+    // FIXME: Pick based on locale
+    equation->priv->source_currency = g_strdup(currency_names[0].short_name);
+    equation->priv->target_currency = g_strdup(currency_names[0].short_name);
+    equation->priv->serializer = mp_serializer_new();
 
     mp_set_from_integer(0, &equation->priv->state.ans);
 }
